/*
 * kier.c
 *
 * Created: 09.01.2017 10:58:05
 *  Author: dsosnowski
 */ 


#define F_CPU 1000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>

#include "defs.h"
#include "ee_mem.h"
#include "wdog.h"


#define SYSTEM_CLOCK			1000000
#define MAIN_DELAY				10
#define	IDLE					0

#define SCANNER_CYCLES_THR		2
#define SYSTEM_STEPS			6

#define TIME_MAX				(2000 / MAIN_DELAY)
#define TIME_DEFAULT			(500 / MAIN_DELAY)

#define SYSTEM_IDLE				0
#define DIRECTION_L				1
#define DIRECTION_R				2
#define DIRECTION_BOTH			3
#define SYSTEM_COMPLETE			4
#define SYSTEM_ABORT			5

#define PROG_START				7
#define PROG_PROC				8



void HardIinit(void);

void InputsScan(void);
uint8_t RiseScan(void);
uint8_t StateScan(void);
uint8_t ProgScan(void);

uint8_t Flasher(uint8_t action);
void Driver(uint8_t state);

void SettRestore(void);
uint8_t TimeProg(uint8_t action);


struct {
	uint16_t time_on;
	uint16_t time_off;

} sett;

struct {
	uint8_t action;
	uint8_t output;
	uint16_t cyclesL;
	uint16_t cyclesR;
	uint16_t cyclesPrg;
	uint16_t cyclesTol;
} system;


//--------------------------------------------------
int main(void)
{
	uint8_t action;
	
	HardIinit();
	
	SettRestore();
	
    while(1)
    {
		//---------------------------------------------------
		InputsScan();
		
		//---------------------------------------------------
		action = RiseScan();
		
		if (action == DIRECTION_L) {
			if (system.action == DIRECTION_L) {
				//abort
				system.action = SYSTEM_ABORT;
			} else {
				//restart
				system.action = DIRECTION_L;
			}
		}
		
		if (action == DIRECTION_R) {
			if (system.action == DIRECTION_R) {
				//abort
				system.action = SYSTEM_ABORT;
			} else {
				//restart
				system.action = DIRECTION_R;
			}
		}
		
		//---------------------------------------------------
		action = StateScan();
		
		if (action > IDLE) {
			if (system.cyclesTol < 0xFFFF) system.cyclesTol++;
			if (system.cyclesTol > (sett.time_off - (sett.time_off / 10))) {
				if (system.action > SYSTEM_IDLE) {
					system.action = SYSTEM_ABORT;
				}			
			}
		} else {
			system.cyclesTol = 0;
		}
		
		//---------------------------------------------------
		action = ProgScan();
		if (action == PROG_START) {
			system.action = SYSTEM_ABORT;
			TimeProg(PROG_START);
		}
		
		if (action == PROG_PROC) {
			system.output = TimeProg(system.action);
			system.action = SYSTEM_IDLE;	
		} else {
			system.output = Flasher(system.action);		
		}
				
		//---------------------------------------------------
		if ((system.output == SYSTEM_COMPLETE) || (system.output == SYSTEM_ABORT)) {
			system.action = SYSTEM_IDLE;
			system.output = 0;
		}

		//---------------------------------------------------
		Driver(system.output);
		
		//---------------------------------------------------
		//_delay_ms(MAIN_DELAY);
		WdtReset();

		set_sleep_mode(SLEEP_MODE_IDLE);
		sleep_enable();
		sleep_cpu();
		sleep_disable();
    }
}

//--------------------------------------------------
void HardIinit(void) {
	PORTB = (1<<PRG_PIN);
	DDRB = (1<<OUTL_PIN | 1<<OUTR_PIN);
	ACSR |= (ACD);
	PRR = (1<<PRTIM1 | 1<<PRUSI | 1<<PRADC);
	WdtSet(7);
		
	TCCR0B = (1<<WGM02);
	TCCR0A = (1<<WGM01 | 1<<WGM00);
	TCNT0 = 0;
	OCR0A = 39;
	TIFR |= (1<<TOV0);
	TIMSK |= (1<<TOIE0);
	TCCR0B = (1<<WGM02 | 1<<CS02);
	sei();
}

//--------------------------------------------------
ISR (TIMER0_OVF_vect) {
	//system.tick++;
}

//--------------------------------------------------
void InputsScan(void) {

	if (INL_GET > 0) {
		if (system.cyclesL <= 0xFFFF) system.cyclesL++;
	} else {
		system.cyclesL = IDLE;
	}

	if (INR_GET > 0) {
		if (system.cyclesR <= 0xFFFF) system.cyclesR++;
	} else {
		system.cyclesR = IDLE;
	}

	if (PRG_GET == 0) {
		if (system.cyclesPrg <= 0xFFFF) system.cyclesPrg++;
	} else {
		system.cyclesPrg = IDLE;
	}
}


//--------------------------------------------------
uint8_t RiseScan(void) {
	uint8_t result = IDLE;

	if (system.output == IDLE) {
		if ((system.cyclesL == SCANNER_CYCLES_THR) && (system.cyclesR == IDLE)) {
			result = DIRECTION_L;
		}
		else if ((system.cyclesR == SCANNER_CYCLES_THR) && (system.cyclesL == IDLE)) {
			result = DIRECTION_R;
		}
	}
	return result;
}

//--------------------------------------------------
uint8_t StateScan(void) {
	uint8_t result = IDLE;

	if (system.output == IDLE) {
		if ((system.cyclesL > SCANNER_CYCLES_THR) && (system.cyclesR == IDLE)) {
			result = DIRECTION_L;
		} else if ((system.cyclesR > SCANNER_CYCLES_THR) && (system.cyclesL == IDLE)) {
			result = DIRECTION_R;
		} else if ((system.cyclesL > SCANNER_CYCLES_THR) && (system.cyclesR > SCANNER_CYCLES_THR)) {
			result = DIRECTION_BOTH;
		}
	}
	return result;
}

//--------------------------------------------------
uint8_t ProgScan(void) {
	uint8_t result = IDLE;

	if (system.cyclesPrg == (500 / MAIN_DELAY)) {
		result = PROG_START;
	}
	if (system.cyclesPrg > (500 / MAIN_DELAY)) {
		result = PROG_PROC;
	}
	return result;
}

//--------------------------------------------------
uint8_t Flasher(uint8_t action) {

static uint8_t step = 0;
static uint16_t counter = 0;
static uint8_t dir = 0;
uint8_t result = 0;

	if ((action == DIRECTION_L) || (action == DIRECTION_R)) {
		if (step == 0) {
			counter = 0;
			step = 1;
			dir = action;
		} else {
			if (action != dir) {
				counter = 0;
				step = 1;
				dir = action;				
			}
		}
	}

	if (action == SYSTEM_ABORT) {
		step = 0;
		counter = 0;
		dir = 0;
	}
		
	if (step > 0) {
		if (step < SYSTEM_STEPS) {
			if ((step & 1) > 0) {
				result = dir;
				if (counter < sett.time_on) {
					counter++;
				} else {
					counter = 0;
					step++;
				}
			} else {
				result = 0;
				if (counter < sett.time_off) {
					counter++;
				} else {
					counter = 0;
					step++;
				}
			}
		} else {
			result = 0;
			if (counter < sett.time_off / 10) {
				counter++;
			} else {
				counter = 0;
				step = 0;
				dir = 0;
				result = SYSTEM_COMPLETE;
			}
		}
	} else {
		result = 0;
	}
	return result;
}

//--------------------------------------------------
void Driver(uint8_t state) {
	if ((state == DIRECTION_L) || (state == DIRECTION_BOTH)) {
		OUTL_SET;
	} else {
		OUTL_OFF;
	}
	if ((state == DIRECTION_R) || (state == DIRECTION_BOTH)) {
		OUTR_SET;
	} else {
		OUTR_OFF;
	}
}

//--------------------------------------------------
void SettRestore(void) {
	sett.time_on = EEread16(4);
	sett.time_off = EEread16(6);
	if (sett.time_on > TIME_MAX) sett.time_on = TIME_DEFAULT;
	if (sett.time_off > TIME_MAX) sett.time_off = TIME_DEFAULT;
}

//--------------------------------------------------
uint8_t TimeProg(uint8_t action) {
	static uint16_t step = 0;
	static uint16_t cycleHi = 0;
	static uint16_t cycleAll = 0;
	
	
	if (action == PROG_START) {
		step = 1;
		cycleHi = 0;
		cycleAll = 0;
	}
	
	if (step > 0) {
		
		if ((action == DIRECTION_L) || (action == DIRECTION_R)) {
			if (step < 0xFFFF) step++;
		}
		
		if (step == 3) {
			
			if (StateScan() > 0) {
				if (cycleHi < 0xFFFF) cycleHi++;
			}
			
			if (cycleAll < 0xFFFF) cycleAll++;
		}
		
		if (step == 4) {
			EEwrite16(4, cycleHi);
			EEwrite16(6, (cycleAll - cycleHi));
			SettRestore();
		}

		if ((step > 4) && (step < (2500 / MAIN_DELAY))){
			if (step < 0xFFFF) step++;
			return DIRECTION_BOTH;		
		}
	}
	return 0;
}